@@ -41,9 +41,9 @@ import gdata.youtube
 
 YOUTUBE_SERVER = 'gdata.youtube.com'
 YOUTUBE_SERVICE = 'youtube'
-YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL = 'https://www.google.com/youtube/accounts/ClientLogin'
+YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL = 'https://www.google.com/accounts/ClientLogin'
 YOUTUBE_SUPPORTED_UPLOAD_TYPES = ('mov', 'avi', 'wmv', 'mpg', 'quicktime',
-                                  'flv')
+                                  'flv', 'mp4', 'x-flv')
 YOUTUBE_QUERY_VALID_TIME_PARAMETERS = ('today', 'this_week', 'this_month',
                                        'all_time')
 YOUTUBE_QUERY_VALID_ORDERBY_PARAMETERS = ('published', 'viewCount', 'rating',
@@ -52,13 +52,13 @@ YOUTUBE_QUERY_VALID_RACY_PARAMETERS = ('include', 'exclude')
 YOUTUBE_QUERY_VALID_FORMAT_PARAMETERS = ('1', '5', '6')
 YOUTUBE_STANDARDFEEDS = ('most_recent', 'recently_featured',
                          'top_rated', 'most_viewed','watch_on_mobile')
-YOUTUBE_UPLOAD_URI = 'http://uploads.gdata.youtube.com/feeds/api/users'
-YOUTUBE_UPLOAD_TOKEN_URI = 'http://gdata.youtube.com/action/GetUploadToken'
-YOUTUBE_VIDEO_URI = 'http://gdata.youtube.com/feeds/api/videos'
-YOUTUBE_USER_FEED_URI = 'http://gdata.youtube.com/feeds/api/users'
-YOUTUBE_PLAYLIST_FEED_URI = 'http://gdata.youtube.com/feeds/api/playlists'
+YOUTUBE_UPLOAD_URI = 'https://uploads.gdata.youtube.com/feeds/api/users'
+YOUTUBE_UPLOAD_TOKEN_URI = 'https://gdata.youtube.com/action/GetUploadToken'
+YOUTUBE_VIDEO_URI = 'https://gdata.youtube.com/feeds/api/videos'
+YOUTUBE_USER_FEED_URI = 'https://gdata.youtube.com/feeds/api/users'
+YOUTUBE_PLAYLIST_FEED_URI = 'https://gdata.youtube.com/feeds/api/playlists'
 
-YOUTUBE_STANDARD_FEEDS = 'http://gdata.youtube.com/feeds/api/standardfeeds'
+YOUTUBE_STANDARD_FEEDS = 'https://gdata.youtube.com/feeds/api/standardfeeds'
 YOUTUBE_STANDARD_TOP_RATED_URI = '%s/%s' % (YOUTUBE_STANDARD_FEEDS, 'top_rated')
 YOUTUBE_STANDARD_MOST_VIEWED_URI = '%s/%s' % (YOUTUBE_STANDARD_FEEDS,
     'most_viewed')
@@ -136,6 +136,8 @@ class YouTubeService(gdata.service.GDataService):
         http://code.google.com/apis/youtube/dashboard to obtain a (free) key.
   """
 
+  ssl = True
+
   def __init__(self, email=None, password=None, source=None,
                server=YOUTUBE_SERVER, additional_headers=None, client_id=None,
                developer_key=None, **kwargs):
@@ -155,18 +157,21 @@ class YouTubeService(gdata.service.GDataService):
       **kwargs: The other parameters to pass to gdata.service.GDataService
           constructor.
     """
-    if developer_key and not client_id:
-      raise YouTubeError('You must also specify the clientId')
 
     gdata.service.GDataService.__init__(
         self, email=email, password=password, service=YOUTUBE_SERVICE,
         source=source, server=server, additional_headers=additional_headers,
         **kwargs)
 
-    if client_id is not None and developer_key is not None:
+    if client_id is not None:
       self.additional_headers['X-Gdata-Client'] = client_id
+
+    if developer_key is not None:
       self.additional_headers['X-GData-Key'] = 'key=%s' % developer_key
 
+    if 'GData-Version' not in self.additional_headers:
+      self.additional_headers['GData-Version'] = 1
+
     self.auth_service_url = YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL
 
   def GetYouTubeVideoFeed(self, uri):
@@ -614,15 +619,15 @@ class YouTubeService(gdata.service.GDataService):
           'body':'`video_entry` must be a gdata.youtube.VideoEntry instance',
           'reason':'Found %s, not VideoEntry' % type(video_entry)
           })
-    majtype, mintype = content_type.split('/')
-
-    try:
-      assert(mintype in YOUTUBE_SUPPORTED_UPLOAD_TYPES)
-    except (ValueError, AssertionError):
-      raise YouTubeError({'status':YOUTUBE_INVALID_CONTENT_TYPE,
-          'body':'This is not a valid content type: %s' % content_type,
-          'reason':'Accepted content types: %s' %
-              ['video/%s' % (t) for t in YOUTUBE_SUPPORTED_UPLOAD_TYPES]})
+    #majtype, mintype = content_type.split('/')
+    #
+    #try:
+    #  assert(mintype in YOUTUBE_SUPPORTED_UPLOAD_TYPES)
+    #except (ValueError, AssertionError):
+    #  raise YouTubeError({'status':YOUTUBE_INVALID_CONTENT_TYPE,
+    #      'body':'This is not a valid content type: %s' % content_type,
+    #      'reason':'Accepted content types: %s' %
+    #          ['video/%s' % (t) for t in YOUTUBE_SUPPORTED_UPLOAD_TYPES]})
 
     if (isinstance(filename_or_handle, (str, unicode)) 
         and os.path.exists(filename_or_handle)):
@@ -632,7 +637,7 @@ class YouTubeService(gdata.service.GDataService):
       import StringIO
       if hasattr(filename_or_handle, 'seek'):
         filename_or_handle.seek(0)
-      file_handle = StringIO.StringIO(filename_or_handle.read())
+      file_handle = filename_or_handle
       name = 'video'
       if hasattr(filename_or_handle, 'name'):
         name = filename_or_handle.name
@@ -739,8 +744,8 @@ class YouTubeService(gdata.service.GDataService):
     for link in video_entry.link:
       if link.rel == 'edit':
         edit_uri = link.href
-    return self.Put(video_entry, uri=edit_uri,
-                    converter=gdata.youtube.YouTubeVideoEntryFromString)
+        return self.Put(video_entry, uri=edit_uri, 
+        	converter=gdata.youtube.YouTubeVideoEntryFromString)
 
   def DeleteVideoEntry(self, video_entry):
     """Deletes a video entry.
@@ -756,7 +761,7 @@ class YouTubeService(gdata.service.GDataService):
     for link in video_entry.link:
       if link.rel == 'edit':
         edit_uri = link.href
-    return self.Delete(edit_uri)
+        return self.Delete(edit_uri)
 
   def AddRating(self, rating_value, video_entry):
     """Add a rating to a video entry.
@@ -1301,7 +1306,7 @@ class YouTubeService(gdata.service.GDataService):
     self.additional_headers['X-Gdata-Client'] = client_id
 
   client_id = property(_GetClientId, _SetClientId,
-                         doc="""The ClientId property""")
+                       doc="""The ClientId property""")
 
   def Query(self, uri):
     """Performs a query and returns a resulting feed or entry.
@@ -1339,12 +1344,12 @@ class YouTubeService(gdata.service.GDataService):
                                      'body': HTTP body of the server response})
     """
     result = self.Query(query.ToUri())
-    if isinstance(query, YouTubeVideoQuery):
-      return gdata.youtube.YouTubeVideoFeedFromString(result.ToString())
-    elif isinstance(query, YouTubeUserQuery):
+    if isinstance(query, YouTubeUserQuery):
       return gdata.youtube.YouTubeUserFeedFromString(result.ToString())
     elif isinstance(query, YouTubePlaylistQuery):
       return gdata.youtube.YouTubePlaylistFeedFromString(result.ToString())
+    elif isinstance(query, YouTubeVideoQuery):
+      return gdata.youtube.YouTubeVideoFeedFromString(result.ToString())
     else:
       return result
 
@@ -1383,17 +1388,20 @@ class YouTubeVideoQuery(gdata.service.Query):
     location: A string of geo coordinates. Note that this is not used when the
         search is performed but rather to filter the returned videos for ones
         that match to the location entered.
+    feed: str (optional) The base URL which is the beginning of the query URL.
+          defaults to 'http://%s/feeds/videos' % (YOUTUBE_SERVER)
   """
 
   def __init__(self, video_id=None, feed_type=None, text_query=None,
-               params=None, categories=None):
+               params=None, categories=None, feed=None):
 
-    if feed_type in YOUTUBE_STANDARDFEEDS:
+    if feed_type in YOUTUBE_STANDARDFEEDS and feed is None:
       feed = 'http://%s/feeds/standardfeeds/%s' % (YOUTUBE_SERVER, feed_type)
-    elif feed_type is 'responses' or feed_type is 'comments' and video_id:
+    elif (feed_type is 'responses' or feed_type is 'comments' and video_id
+          and feed is None):
       feed = 'http://%s/feeds/videos/%s/%s' % (YOUTUBE_SERVER, video_id,
                                                feed_type)
-    else:
+    elif feed is None:
       feed = 'http://%s/feeds/videos' % (YOUTUBE_SERVER)
 
     gdata.service.Query.__init__(self, feed, text_query=text_query,
@@ -1536,7 +1544,7 @@ class YouTubeUserQuery(YouTubeVideoQuery):
     else:
       feed = "http://%s/feeds/users" % (YOUTUBE_SERVER)
 
-    YouTubeVideoQuery.__init__(self, feed, text_query=text_query,
+    YouTubeVideoQuery.__init__(self, feed=feed, text_query=text_query,
                                params=params, categories=categories)
 
 
@@ -1556,5 +1564,5 @@ class YouTubePlaylistQuery(YouTubeVideoQuery):
     else:
       feed = "http://%s/feeds/playlists" % (YOUTUBE_SERVER)
 
-    YouTubeVideoQuery.__init__(self, feed, text_query=text_query,
+    YouTubeVideoQuery.__init__(self, feed=feed, text_query=text_query,
                                params=params, categories=categories)
